
# This file was generated by 'versioneer.py' (0.18) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2024-12-11T17:03:00-0500",
 "dirty": false,
 "error": null,
 "full-revisionid": "b2c8d3f1c59ea8273e29b0929b451f3d4b44c0ab",
 "version": "2024.12.11"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
